@@ -73,6 +73,14 @@ ipyvue.register_component_from_file(None, 'j-plugin-section-header',
                                     os.path.join(os.path.dirname(__file__),
                                                  'components/plugin_section_header.vue'))
 
+ipyvue.register_component_from_file(None, 'plugin-subset-select',
+                                    os.path.join(os.path.dirname(__file__),
+                                                 'components/plugin_subset_select.vue'))
+
+ipyvue.register_component_from_file(None, 'plugin-viewer-select',
+                                    os.path.join(os.path.dirname(__file__),
+                                                 'components/plugin_viewer_select.vue'))
+
 # Register pure vue component. This allows us to do recursive component instantiation only in the
 # vue component file
 ipyvue.register_component_from_file('g-viewer-tab', "container.vue", __file__)
@@ -302,6 +310,9 @@ class Application(VuetifyTemplate, HubListener):
         any components are compatible with already loaded data. If so, link
         them so that they can be displayed on the same profile1D plot.
         """
+        if self.config == 'imviz':  # Imviz does its own thing
+            return
+
         new_len = len(self.data_collection)
 
         # Allow for batch linking of data in the parser rather than on
@@ -311,16 +322,31 @@ class Application(VuetifyTemplate, HubListener):
 
         # Can't link if there's no world_component_ids
         wc_new = self.data_collection[-1].world_component_ids
-        if wc_new == []:
-            return
+        pc_new = self.data_collection[-1].pixel_component_ids
 
         # Link to the first dataset with compatible coordinates
         for i in range(new_len - 1):
             wc_old = self.data_collection[i].world_component_ids
+            pc_old = self.data_collection[i].pixel_component_ids
+
             if wc_old == []:
                 continue
             else:
-                self.data_collection.add_link(LinkSame(wc_old[0], wc_new[0]))
+                # Link cubes to each other
+                if len(pc_old) > 2 and len(pc_new) > 2:
+                    links = [LinkSame(pc_old[1], pc_new[1]),
+                             LinkSame(pc_old[2], pc_new[2]),
+                             LinkSame(wc_old[0], wc_new[0])]
+
+                # If data is not a cube, use old linking method
+                elif len(wc_new) > 0:
+                    links = [LinkSame(wc_old[0], wc_new[0])]
+
+                else:
+                    break
+
+                self.data_collection.add_link(links)
+
                 break
 
     def load_data(self, file_obj, parser_reference=None, **kwargs):
@@ -1086,18 +1112,6 @@ class Application(VuetifyTemplate, HubListener):
 
         self._update_selected_data_items(viewer_id, selected_items)
 
-    def vue_save_figure(self, event):
-        """
-        Callback for save figure events in the front end viewer toolbars. Uses
-        the bqplot.Figure save methods.
-        """
-        viewer_id, filetype = event['id'], event['filetype']
-        viewer = self._viewer_store[viewer_id]
-        if filetype == "png":
-            viewer.figure.save_png()
-        elif filetype == "svg":
-            viewer.figure.save_svg()
-
     def vue_close_snackbar_message(self, event):
         """
         Callback to close a message in the snackbar when the "close"
@@ -1165,6 +1179,15 @@ class Application(VuetifyTemplate, HubListener):
                     and active_data._preferred_translation is not None):
                 self._set_plot_axes_labels(viewer_id)
 
+            # Make everything visible again in Imviz.
+            if self.config == 'imviz':
+                from jdaviz.configs.imviz.helper import layer_is_image_data
+
+                for lyr in viewer.state.layers:
+                    if layer_is_image_data(lyr.layer):
+                        lyr.visible = True
+                viewer.on_limits_change()  # Trigger compass redraw
+
     def _on_data_added(self, msg):
         """
         Callback for when data is added to the internal ``DataCollection``.
@@ -1276,10 +1299,16 @@ class Application(VuetifyTemplate, HubListener):
             n = self._next_viewer_num(pfx)
             vid = f"{pfx}-{n}"
 
+        # There is a viewer.LABEL inherited from glue-jupyter but there was
+        # objection in using it here because it is not hidden, so we use our
+        # own attribute instead.
+        viewer._reference_id = vid  # For reverse look-up
+
         return {
             'id': vid,
             'name': name or vid,
             'widget': "IPY_MODEL_" + viewer.figure_widget.model_id,
+            'toolbar_nested': "IPY_MODEL_" + viewer.toolbar_nested.model_id if viewer.toolbar_nested else '',  # noqa
             'tools': "IPY_MODEL_" + viewer.toolbar_selection_tools.model_id,
             'tools_open': False,
             'layer_options': "IPY_MODEL_" + viewer.layer_options.model_id,
