@@ -17,7 +17,6 @@ def test_moment_calculation(cubeviz_helper, spectrum1d_cube, tmpdir):
     mm._on_data_updated(None)
 
     mm._on_data_selected({'new': 'test[FLUX]'})
-    mm._on_subset_selected({'new': None})
 
     mm.n_moment = 0  # Collapsed sum, will get back 2D spatial image
     mm.vue_calculate_moment()
@@ -25,6 +24,8 @@ def test_moment_calculation(cubeviz_helper, spectrum1d_cube, tmpdir):
     assert mm.moment_available
     assert dc[1].label == 'Moment 0: test[FLUX]'
 
+    assert len(dc.links) == 8
+
     result = dc[1].get_object(cls=CCDData)
     assert result.shape == (2, 4)  # Cube shape is (2, 2, 4)
 
@@ -39,3 +40,10 @@ def test_moment_calculation(cubeviz_helper, spectrum1d_cube, tmpdir):
     # Does not allow overwrite.
     with pytest.raises(OSError, match='already exists'):
         mm.vue_save_as_fits()
+
+    mm.n_moment = 1
+    mm.vue_calculate_moment()
+
+    assert dc[2].label == 'Moment 1: test[FLUX]'
+
+    assert len(dc.links) == 10
