@@ -2,9 +2,15 @@ import astropy.modeling.models as models
 import astropy.modeling.parameters as params
 import astropy.units as u
 import numpy as np
+import pytest
 from specutils.spectra import Spectrum1D
 
-from .. import fitting_backend as fb
+from glue.core import Data
+
+from jdaviz import Application
+from jdaviz.configs.default.plugins.model_fitting import fitting_backend as fb
+from jdaviz.configs.default.plugins.model_fitting import initializers
+from jdaviz.configs.default.plugins.model_fitting.model_fitting import ModelFitting
 
 
 SPECTRUM_SIZE = 200  # length of spectrum
@@ -24,6 +30,55 @@ def build_spectrum(sigma=0.1):
     return x, y + noise
 
 
+def test_model_params():
+    # expected model parameters:
+    model_parameters = {"Gaussian1D": ["amplitude", "stddev", "mean"],
+                        "Const1D": ["amplitude"],
+                        "Linear1D": ["slope", "intercept"],
+                        "Polynomial1D": ["c0", "c1"],
+                        "PowerLaw1D": ["amplitude", "x_0", "alpha"],
+                        "Lorentz1D": ["amplitude", "x_0", "fwhm"],
+                        "Voigt1D": ["x_0", "amplitude_L", "fwhm_L", "fwhm_G"],
+                        "BlackBody": ["temperature", "scale"],
+                        }
+
+    for model_name in initializers.MODELS.keys():
+        if model_name not in model_parameters.keys():
+            # this would be caught later by the assertion anyways,
+            # but raising an error will be more clear that the
+            # test needs to be updated rather than the code breaking
+            raise ValueError(f"{model_name} not in test dictionary of expected parameters")
+        expected_params = model_parameters.get(model_name, [])
+        params = initializers.get_model_parameters(model_name)
+        assert len(params) == len(expected_params)
+        assert np.all([p in expected_params for p in params])
+
+
+@pytest.mark.filterwarnings('ignore')
+def test_model_ids(spectral_cube_wcs):
+    app = Application()
+    dc = app.data_collection
+    dc.append(Data(x=np.ones((3, 4, 5)), label='test', coords=spectral_cube_wcs))
+
+    plugin = ModelFitting(app=app)
+
+    plugin.data_selected = 'test'
+    plugin.component_models = [{'id': 'valid_string_already_exists'}]
+    plugin.temp_model = 'Linear1D'
+
+    with pytest.raises(
+            ValueError,
+            match="model component ID valid_string_already_exists already in use"):
+        plugin.temp_name = 'valid_string_already_exists'
+        plugin.vue_add_model({})
+
+    with pytest.raises(
+            ValueError,
+            match="invalid model component ID invalid-string"):
+        plugin.temp_name = 'invalid-string'
+        plugin.vue_add_model({})
+
+
 def test_fitting_backend():
     np.random.seed(42)
 
@@ -39,7 +94,7 @@ def test_fitting_backend():
     model_list = [g1f, g2f, g3f, zero_level]
     expression = "g1 + g2 + g3 + const1d"
 
-    # Returns the intial model
+    # Returns the initial model
     fm, fitted_spectrum = fb.fit_model_to_spectrum(spectrum, model_list, expression,
                                                    run_fitter=False)
 
@@ -57,6 +112,9 @@ def test_fitting_backend():
     assert np.allclose(fm.parameters, parameters_expected, atol=1e-5)
 
 
+# When pytest turns warnings into errors, this silently fails with
+# len(fitted_parameters) == 0
+@pytest.mark.filterwarnings('ignore')
 def test_cube_fitting_backend():
     np.random.seed(42)
 
