@@ -26,14 +26,27 @@ class ImvizImageView(BqplotImageView, AstrowidgetsImageViewerMixin, JdavizViewer
              'bqplot:panzoom', 'jdaviz:panzoommatch',
              'jdaviz:contrastbias', 'jdaviz:blinkonce',
              'bqplot:rectangle', 'bqplot:circle', 'bqplot:ellipse']
+
+    # categories: zoom resets, zoom, pan, subset, select tools, shortcuts
+    tools_nested = [
+                    ['bqplot:home'],
+                    ['jdaviz:boxzoom', 'jdaviz:boxzoommatch'],
+                    ['bqplot:panzoom', 'jdaviz:panzoommatch'],
+                    ['bqplot:circle', 'bqplot:rectangle', 'bqplot:ellipse'],
+                    ['jdaviz:blinkonce', 'jdaviz:contrastbias'],
+                    ['jdaviz:sidebar_plot', 'jdaviz:sidebar_export', 'jdaviz:sidebar_compass']
+                ]
+
     default_class = None
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.init_astrowidgets_api()
+        self._initialize_toolbar_nested()
 
         self.label_mouseover = None
         self.compass = None
+        self.line_profile_xy = None
 
         self.add_event_callback(self.on_mouse_or_key_event, events=['mousemove', 'mouseenter',
                                                                     'mouseleave', 'keydown'])
@@ -70,6 +83,13 @@ class ImvizImageView(BqplotImageView, AstrowidgetsImageViewerMixin, JdavizViewer
             else:
                 return
 
+        if self.line_profile_xy is None:
+            try:
+                self.line_profile_xy = self.session.jdaviz_app.get_tray_item_from_name(
+                    'imviz-line-profile-xy')
+            except KeyError:  # pragma: no cover
+                return
+
         if data['event'] == 'mousemove':
             # Display the current cursor coordinates (both pixel and world) as
             # well as data values. For now we use the first dataset in the
@@ -132,12 +152,22 @@ class ImvizImageView(BqplotImageView, AstrowidgetsImageViewerMixin, JdavizViewer
             self.label_mouseover.reset_coords_display()
             self.label_mouseover.value = ""
 
-        elif data['event'] == 'keydown' and data['key'] == 'b':
-            self.blink_once()
+        elif data['event'] == 'keydown':
+            key_pressed = data['key']
 
-            # Also update the coordinates display.
-            data['event'] = 'mousemove'
-            self.on_mouse_or_key_event(data)
+            if key_pressed == 'b':
+                self.blink_once()
+
+                # Also update the coordinates display.
+                data['event'] = 'mousemove'
+                self.on_mouse_or_key_event(data)
+
+            elif key_pressed == 'l':
+                # Same data as mousemove above.
+                self.line_profile_xy.selected_x = data['domain']['x']
+                self.line_profile_xy.selected_y = data['domain']['y']
+                self.line_profile_xy.selected_viewer = self.reference_id
+                self.line_profile_xy.vue_draw_plot()
 
     def blink_once(self):
         # Simple blinking of images - this will make it so that only one
@@ -175,6 +205,16 @@ class ImvizImageView(BqplotImageView, AstrowidgetsImageViewerMixin, JdavizViewer
                 # We can display the active data label in Compass plugin.
                 self.set_compass(self.state.layers[next_layer].layer)
 
+                # Update line profile plots too.
+                if self.line_profile_xy is None:
+                    try:
+                        self.line_profile_xy = self.session.jdaviz_app.get_tray_item_from_name(
+                            'imviz-line-profile-xy')
+                    except KeyError:  # pragma: no cover
+                        return
+                self.line_profile_xy.selected_viewer = self.reference_id
+                self.line_profile_xy.vue_draw_plot()
+
     def on_limits_change(self, *args):
         try:
             i = get_top_layer_index(self)
@@ -184,28 +224,37 @@ class ImvizImageView(BqplotImageView, AstrowidgetsImageViewerMixin, JdavizViewer
             return
         self.set_compass(self.state.layers[i].layer)
 
+    def _get_zoom_limits(self, image):
+        """Return ``(x_min, y_min, x_max, y_max)`` for given image.
+        This is needed because viewer values are only based on reference
+        image, which can be inaccurate if given image is dithered and
+        they are linked by WCS.
+        """
+        if data_has_valid_wcs(image) and self.get_link_type(image.label) == 'wcs':
+            # Convert X,Y from reference data to the one we are actually seeing.
+            x = image.coords.world_to_pixel(self.state.reference_data.coords.pixel_to_world(
+                    (self.state.x_min, self.state.x_max), (self.state.y_min, self.state.y_max)))
+            zoom_limits = (x[0][0], x[1][0], x[0][1], x[1][1])
+        else:
+            zoom_limits = (self.state.x_min, self.state.y_min, self.state.x_max, self.state.y_max)
+        return zoom_limits
+
     def set_compass(self, image):
         """Update the Compass plugin with info from the given image Data object."""
         if self.compass is None:  # Maybe another viewer has it
             return
 
-        zoom_limits = (self.state.x_min, self.state.y_min, self.state.x_max, self.state.y_max)
-        if data_has_valid_wcs(image):
-            wcs = image.coords
+        zoom_limits = self._get_zoom_limits(image)
 
-            # Convert X,Y from reference data to the one we are actually seeing.
-            if self.get_link_type(image.label) == 'wcs':
-                x = wcs.world_to_pixel(self.state.reference_data.coords.pixel_to_world(
-                        (self.state.x_min, self.state.x_max), (self.state.y_min, self.state.y_max)))
-                zoom_limits = (x[0][0], x[1][0], x[0][1], x[1][1])
-        else:
-            wcs = None
-
-        arr = image[image.main_components[0]]
+        # Downsample input data to about 400px (as per compass.vue) for performance.
+        xstep = max(1, round(image.shape[1] / 400))
+        ystep = max(1, round(image.shape[0] / 400))
+        arr = image[image.main_components[0]][::ystep, ::xstep]
         vmin, vmax = PercentileInterval(95).get_limits(arr)
         norm = ImageNormalize(vmin=vmin, vmax=vmax, stretch=LinearStretch())
         self.compass.draw_compass(image.label, wcs_utils.draw_compass_mpl(
-            arr, wcs, show=False, zoom_limits=zoom_limits, norm=norm))
+            arr, orig_shape=image.shape, wcs=image.coords, show=False, zoom_limits=zoom_limits,
+            norm=norm))
 
     def set_plot_axes(self):
         self.figure.axes[1].tick_format = None
