@@ -13,7 +13,6 @@ from regions import CirclePixelRegion, RectanglePixelRegion
 from skimage.io import imsave
 
 from jdaviz.configs.imviz.helper import split_filename_with_fits_ext
-from jdaviz.configs.imviz.plugins.aper_phot_simple.aper_phot_simple import SimpleAperturePhotometry
 from jdaviz.configs.imviz.plugins.parsers import (
     parse_data, _validate_fits_image2d, _validate_bunit, _parse_image)
 
@@ -70,19 +69,18 @@ class TestParseImage:
     def test_hdulist_no_image(self, imviz_helper):
         hdulist = fits.HDUList([fits.PrimaryHDU()])
         with pytest.raises(ValueError, match='does not have any FITS image'):
-            parse_data(imviz_helper.app, hdulist, show_in_viewer=False)
+            parse_data(imviz_helper.app, hdulist)
 
     @pytest.mark.parametrize('some_obj', (WCS(), [[1, 2], [3, 4]]))
     def test_invalid_file_obj(self, imviz_helper, some_obj):
         with pytest.raises(NotImplementedError, match='Imviz does not support'):
-            parse_data(imviz_helper.app, some_obj, show_in_viewer=False)
+            parse_data(imviz_helper.app, some_obj)
 
     def test_parse_numpy_array_1d_2d(self, imviz_helper):
         with pytest.raises(ValueError, match='Imviz cannot load this array with ndim=1'):
-            parse_data(imviz_helper.app, np.zeros(2), show_in_viewer=False)
+            parse_data(imviz_helper.app, np.zeros(2))
 
-        parse_data(imviz_helper.app, np.zeros((2, 2)), data_label='some_array',
-                   show_in_viewer=False)
+        imviz_helper.load_data(np.zeros((2, 2)), data_label='some_array', show_in_viewer=False)
         data = imviz_helper.app.data_collection[0]
         comp = data.get_component('DATA')
         assert data.label == 'some_array'
@@ -134,10 +132,10 @@ class TestParseImage:
 
     def test_parse_nddata_simple(self, imviz_helper):
         with pytest.raises(ValueError, match='Imviz cannot load this NDData with ndim=1'):
-            parse_data(imviz_helper.app, NDData([1, 2, 3, 4]), show_in_viewer=False)
+            parse_data(imviz_helper.app, NDData([1, 2, 3, 4]))
 
         ndd = NDData([[1, 2], [3, 4]])
-        parse_data(imviz_helper.app, ndd, data_label='some_data', show_in_viewer=False)
+        imviz_helper.load_data(ndd, data_label='some_data', show_in_viewer=False)
         data = imviz_helper.app.data_collection[0]
         comp = data.get_component('DATA')
         assert data.label == 'some_data[DATA]'
@@ -152,7 +150,7 @@ class TestParseImage:
          (NDData([[1, 2], [3, 4]], uncertainty=StdDevUncertainty([[0.1, 0.2], [0.3, 0.4]])),
           ['DATA', 'UNCERTAINTY'])])
     def test_parse_nddata_with_one_only(self, imviz_helper, ndd, attributes):
-        parse_data(imviz_helper.app, ndd, data_label='some_data', show_in_viewer=False)
+        imviz_helper.load_data(ndd, data_label='some_data', show_in_viewer=False)
         for i, attrib in enumerate(attributes):
             data = imviz_helper.app.data_collection[i]
             comp = data.get_component(attrib)
@@ -165,7 +163,7 @@ class TestParseImage:
         ndd = NDData([[1, 2], [3, 4]], mask=[[True, False], [False, False]],
                      uncertainty=StdDevUncertainty([[0.1, 0.2], [0.3, 0.4]]),
                      unit=u.MJy/u.sr, wcs=WCS(naxis=2), meta={'name': 'my_ndd'})
-        parse_data(imviz_helper.app, ndd, data_label='some_data', show_in_viewer=False)
+        imviz_helper.load_data(ndd, data_label='some_data', show_in_viewer=False)
         for i, attrib in enumerate(['DATA', 'MASK', 'UNCERTAINTY']):
             data = imviz_helper.app.data_collection[i]
             comp = data.get_component(attrib)
@@ -191,7 +189,7 @@ class TestParseImage:
         filename = str(tmp_path / f'myimage.{format}')
         imsave(filename, a)
 
-        parse_data(imviz_helper.app, filename, show_in_viewer=False)
+        imviz_helper.load_data(filename, show_in_viewer=False)
         data = imviz_helper.app.data_collection[0]
         assert data.label == 'myimage'
         assert data.shape == (10, 10)
@@ -238,7 +236,7 @@ class TestParseImage:
         filename = download_file(self.jwst_asdf_url_1, cache=True)
 
         # Default behavior: Science image
-        parse_data(imviz_helper.app, filename, show_in_viewer=True)
+        imviz_helper.load_data(filename)
         data = imviz_helper.app.data_collection[0]
         comp = data.get_component('DATA')
         assert data.label == 'contents[DATA]'  # download_file returns cache loc
@@ -262,7 +260,7 @@ class TestParseImage:
         assert isinstance(subsets['Subset 2'], RectanglePixelRegion)
 
         # Test simple aperture photometry plugin.
-        phot_plugin = SimpleAperturePhotometry(app=imviz_helper.app)
+        phot_plugin = imviz_helper.app.get_tray_item_from_name('imviz-aper-phot-simple')
         phot_plugin._on_viewer_data_changed()
         phot_plugin.data_selected = 'contents[DATA]'
         phot_plugin.subset_selected = 'Subset 1'
@@ -276,33 +274,43 @@ class TestParseImage:
         phot_plugin.flux_scaling = 1  # Simple mag, no zeropoint
         phot_plugin.vue_do_aper_phot()
         tbl = imviz_helper.get_aperture_photometry_results()
-        assert_quantity_allclose(tbl[0]['xcenter'], 970.95 * u.pix)
-        assert_quantity_allclose(tbl[0]['ycenter'], 1116.05 * u.pix)
-        sky = tbl[0]['sky_center']
+        assert_quantity_allclose(tbl[0]['xcentroid'], 970.935492 * u.pix)
+        assert_quantity_allclose(tbl[0]['ycentroid'], 1116.967619 * u.pix)
+        sky = tbl[0]['sky_centroid']
         assert_allclose(sky.ra.deg, 80.48419863)
-        assert_allclose(sky.dec.deg, -69.49460838)
+        assert_allclose(sky.dec.deg, -69.494592)
         data_unit = u.MJy / u.sr
         assert_quantity_allclose(tbl[0]['background'], 0.1741226315498352 * data_unit)
-        assert_quantity_allclose(tbl[0]['npix'], 111.22023392 * u.pix)
-        assert_quantity_allclose(tbl[0]['aperture_sum'], 4.989882e-09 * u.MJy)
-        assert_quantity_allclose(tbl[0]['pixarea_tot'], 1.0384377922763469e-11 * u.sr)
-        assert_quantity_allclose(tbl[0]['aperture_sum_counts'], 132061.576643 * u.count)
+        assert_quantity_allclose(tbl[0]['sum'], 4.989882e-09 * u.MJy)
+        assert_quantity_allclose(tbl[0]['sum_aper_area'], 111.220234 * (u.pix * u.pix))
+        assert_quantity_allclose(tbl[0]['pixarea_tot'], 1.038438e-11 * u.sr, atol=1e-15 * u.sr)
+        assert_quantity_allclose(tbl[0]['aperture_sum_counts'], 132061.576643 * u.count, rtol=1e-6)
         assert_quantity_allclose(tbl[0]['aperture_sum_counts_err'], 363.402775 * u.count)
         assert_quantity_allclose(tbl[0]['counts_fac'], 0.0036385915646798953 * (data_unit / u.ct))
         assert_quantity_allclose(tbl[0]['aperture_sum_mag'], -6.704274 * u.mag)
         assert_quantity_allclose(tbl[0]['flux_scaling'], 1 * data_unit)
-        assert_quantity_allclose(tbl[0]['mean'], 4.391718 * data_unit)
-        assert_quantity_allclose(tbl[0]['stddev'], 15.618626488031158 * data_unit)
-        assert_quantity_allclose(tbl[0]['median'], 0.482972 * data_unit, rtol=1e-5)
         assert_quantity_allclose(tbl[0]['min'], 0.041017 * data_unit, atol=1e-5 * data_unit)
         assert_quantity_allclose(tbl[0]['max'], 138.923752 * data_unit, rtol=1e-5)
+        assert_quantity_allclose(tbl[0]['mean'], 4.391718 * data_unit)
+        assert_quantity_allclose(tbl[0]['median'], 0.482972 * data_unit, rtol=1e-5)
+        assert_quantity_allclose(tbl[0]['mode'], -7.33452046 * data_unit)
+        assert_quantity_allclose(tbl[0]['std'], 15.618626488031158 * data_unit)
+        assert_quantity_allclose(tbl[0]['mad_std'], 0.47892631 * data_unit)
+        assert_quantity_allclose(tbl[0]['var'], 243.9415 * (data_unit * data_unit))
+        assert_quantity_allclose(tbl[0]['biweight_location'], 0.40664572 * data_unit)
+        assert_quantity_allclose(tbl[0]['biweight_midvariance'], 0.27319583 * (data_unit * data_unit))  # noqa
+        assert_quantity_allclose(tbl[0]['fwhm'], 2.8691718 * u.pix)
+        assert_quantity_allclose(tbl[0]['semimajor_sigma'], 1.22308648 * u.pix)
+        assert_quantity_allclose(tbl[0]['semiminor_sigma'], 1.21374578 * u.pix)
+        assert_quantity_allclose(tbl[0]['orientation'], -27.220559 * u.deg)
+        assert_quantity_allclose(tbl[0]['eccentricity'], 0.12335181)
 
         # --- Back to parser testing below. ---
 
         # Request specific extension (name + ver, but ver is not used), use given label
-        parse_data(imviz_helper.app, filename, ext=('DQ', 42),
-                   data_label='jw01072001001_01101_00001_nrcb1_cal',
-                   show_in_viewer=False)
+        imviz_helper.load_data(filename, ext=('DQ', 42),
+                               data_label='jw01072001001_01101_00001_nrcb1_cal',
+                               show_in_viewer=False)
         data = imviz_helper.app.data_collection[1]
         comp = data.get_component('DQ')
         assert data.label == 'jw01072001001_01101_00001_nrcb1_cal[DQ]'
@@ -311,9 +319,9 @@ class TestParseImage:
 
         # Pass in HDUList directly + ext (name only), use given label
         with fits.open(filename) as pf:
-            parse_data(imviz_helper.app, pf, ext='SCI',
-                       data_label='jw01072001001_01101_00001_nrcb1_cal',
-                       show_in_viewer=False)
+            imviz_helper.load_data(pf, ext='SCI',
+                                   data_label='jw01072001001_01101_00001_nrcb1_cal',
+                                   show_in_viewer=False)
             data = imviz_helper.app.data_collection[2]
             comp = data.get_component('DATA')  # SCI = DATA
             assert data.label == 'jw01072001001_01101_00001_nrcb1_cal[DATA]'
@@ -322,17 +330,17 @@ class TestParseImage:
 
             # Test duplicate label functionality
             imviz_helper.app.data_collection.clear()
-            parse_data(imviz_helper.app, pf, ext='SCI', show_in_viewer=False, data_label='TEST')
+            imviz_helper.load_data(pf, ext='SCI', data_label='TEST', show_in_viewer=False)
             data = imviz_helper.app.data_collection[0]
             assert data.label.endswith('[DATA]')
 
-            parse_data(imviz_helper.app, pf, ext='SCI', show_in_viewer=False, data_label='TEST')
+            imviz_helper.load_data(pf, ext='SCI', data_label='TEST', show_in_viewer=False)
             data = imviz_helper.app.data_collection[1]
             assert data.label.endswith('[DATA]_2')
 
             # Load all extensions
             imviz_helper.app.data_collection.clear()
-            parse_data(imviz_helper.app, pf, ext='*', show_in_viewer=False)
+            imviz_helper.load_data(pf, ext='*', show_in_viewer=False)
             data = imviz_helper.app.data_collection
             assert len(data.labels) == 7
             assert data.labels[0].endswith('[DATA]')
@@ -345,15 +353,14 @@ class TestParseImage:
 
         # Invalid ASDF attribute (extension)
         with pytest.raises(KeyError, match='does_not_exist'):
-            parse_data(imviz_helper.app, filename, ext='DOES_NOT_EXIST',
-                       data_label='foo', show_in_viewer=False)
+            parse_data(imviz_helper.app, filename, ext='DOES_NOT_EXIST', data_label='foo')
 
     @pytest.mark.remote_data
     def test_parse_jwst_niriss_grism(self, imviz_helper):
         """No valid image GWCS for Imviz, will fall back to FITS loading without WCS."""
         filename = download_file(self.jwst_asdf_url_2, cache=True)
 
-        parse_data(imviz_helper.app, filename, show_in_viewer=False)
+        imviz_helper.load_data(filename, show_in_viewer=False)
         data = imviz_helper.app.data_collection[0]
         comp = data.get_component('SCI,1')
         assert data.label == 'contents[SCI,1]'  # download_file returns cache loc
@@ -369,7 +376,7 @@ class TestParseImage:
         filename = download_file(url, cache=True)
 
         # Default behavior: Load first image
-        parse_data(imviz_helper.app, filename, show_in_viewer=True)
+        imviz_helper.load_data(filename)
         data = imviz_helper.app.data_collection[0]
         comp = data.get_component('SCI,1')
         assert data.label == 'contents[SCI,1]'  # download_file returns cache loc
@@ -385,7 +392,7 @@ class TestParseImage:
         imviz_helper._apply_interactive_region('bqplot:ellipse',
                                                (1465, 2541),
                                                (1512, 2611))  # Galaxy
-        phot_plugin = SimpleAperturePhotometry(app=imviz_helper.app)
+        phot_plugin = imviz_helper.app.get_tray_item_from_name('imviz-aper-phot-simple')
         phot_plugin._on_viewer_data_changed()
         phot_plugin.data_selected = 'contents[SCI,1]'
         phot_plugin.subset_selected = 'Subset 1'
@@ -393,30 +400,40 @@ class TestParseImage:
         assert_allclose(phot_plugin.pixel_area, 0.0025)  # Not used but still auto-populated
         phot_plugin.vue_do_aper_phot()
         tbl = imviz_helper.get_aperture_photometry_results()
-        assert_quantity_allclose(tbl[0]['xcenter'], 1488.5 * u.pix)
-        assert_quantity_allclose(tbl[0]['ycenter'], 2576 * u.pix)
-        sky = tbl[0]['sky_center']
-        assert_allclose(sky.ra.deg, 3.6840882015888323, rtol=1e-5)
-        assert_allclose(sky.dec.deg, 10.802065746813046, rtol=1e-5)
+        assert_quantity_allclose(tbl[0]['xcentroid'], 1487.60825422 * u.pix)
+        assert_quantity_allclose(tbl[0]['ycentroid'], 2573.83983184 * u.pix)
+        sky = tbl[0]['sky_centroid']
+        assert_allclose(sky.ra.deg, 3.684062989070131, rtol=1e-5)
+        assert_allclose(sky.dec.deg, 10.802045612042956, rtol=1e-5)
         data_unit = u.electron / u.s
         assert_quantity_allclose(tbl[0]['background'], 0.0014 * data_unit)
-        assert_quantity_allclose(tbl[0]['npix'], 645.98998939 * u.pix)
-        assert_quantity_allclose(tbl[0]['aperture_sum'], 78.352207 * data_unit)
+        assert_quantity_allclose(tbl[0]['sum'], 112.680738 * data_unit)
+        assert_quantity_allclose(tbl[0]['sum_aper_area'], 2583.959958 * (u.pix * u.pix))
         assert_array_equal(tbl[0]['pixarea_tot'], None)
         assert_array_equal(tbl[0]['aperture_sum_counts'], None)
         assert_array_equal(tbl[0]['aperture_sum_counts_err'], None)
         assert_array_equal(tbl[0]['counts_fac'], None)
         assert_array_equal(tbl[0]['aperture_sum_mag'], None)
         assert_array_equal(tbl[0]['flux_scaling'], None)
-        assert_quantity_allclose(tbl[0]['mean'], 0.120531 * data_unit, rtol=1e-5)
-        assert_quantity_allclose(tbl[0]['stddev'], 0.174565 * data_unit, rtol=1e-5)
-        assert_quantity_allclose(tbl[0]['median'], 0.068294 * data_unit, rtol=1e-5)
-        assert_quantity_allclose(tbl[0]['min'], -0.004208 * data_unit, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['min'], -0.024501 * data_unit, rtol=1e-3)
         assert_quantity_allclose(tbl[0]['max'], 1.625122 * data_unit, rtol=1e-5)
+        assert_quantity_allclose(tbl[0]['mean'], 0.043656 * data_unit, rtol=1e-5)
+        assert_quantity_allclose(tbl[0]['median'], 0.02161 * data_unit, rtol=1e-5)
+        assert_quantity_allclose(tbl[0]['mode'], -0.02248222 * data_unit, rtol=1e-5)
+        assert_quantity_allclose(tbl[0]['std'], 0.099902 * data_unit, rtol=1e-5)
+        assert_quantity_allclose(tbl[0]['mad_std'], 0.02354494 * data_unit, rtol=1e-5)
+        assert_quantity_allclose(tbl[0]['var'], 0.00998049 * (data_unit * data_unit), rtol=1e-5)
+        assert_quantity_allclose(tbl[0]['biweight_location'], 0.02106674 * data_unit, rtol=1e-5)
+        assert_quantity_allclose(tbl[0]['biweight_midvariance'], 0.00062667 * (data_unit * data_unit), rtol=1e-5)  # noqa
+        assert_quantity_allclose(tbl[0]['fwhm'], 21.26519725 * u.pix, rtol=1e-5)
+        assert_quantity_allclose(tbl[0]['semimajor_sigma'], 9.75795611 * u.pix, rtol=1e-5)
+        assert_quantity_allclose(tbl[0]['semiminor_sigma'], 8.23905782 * u.pix, rtol=1e-5)
+        assert_quantity_allclose(tbl[0]['orientation'], 79.89755434 * u.deg, rtol=1e-5)
+        assert_quantity_allclose(tbl[0]['eccentricity'], 0.5358037, rtol=1e-5)
 
         # Request specific extension (name only), use given label
-        parse_data(imviz_helper.app, filename, ext='CTX',
-                   data_label='jclj01010_drz', show_in_viewer=False)
+        imviz_helper.load_data(filename, ext='CTX', data_label='jclj01010_drz',
+                               show_in_viewer=False)
         data = imviz_helper.app.data_collection[1]
         comp = data.get_component('CTX,1')
         assert data.label == 'jclj01010_drz[CTX,1]'
@@ -424,8 +441,8 @@ class TestParseImage:
         assert comp.units == ''  # BUNIT is not set
 
         # Request specific extension (name + ver), use given label
-        parse_data(imviz_helper.app, filename, ext=('WHT', 1),
-                   data_label='jclj01010_drz', show_in_viewer=False)
+        imviz_helper.load_data(filename, ext=('WHT', 1), data_label='jclj01010_drz',
+                               show_in_viewer=False)
         data = imviz_helper.app.data_collection[2]
         comp = data.get_component('WHT,1')
         assert data.label == 'jclj01010_drz[WHT,1]'
@@ -435,21 +452,21 @@ class TestParseImage:
         # Pass in file obj directly
         with fits.open(filename) as pf:
             # Default behavior: Load first image
-            parse_data(imviz_helper.app, pf, show_in_viewer=False)
+            imviz_helper.load_data(pf, show_in_viewer=False)
             data = imviz_helper.app.data_collection[3]
             assert data.label.startswith('imviz_data|') and data.label.endswith('[SCI,1]')
             assert_allclose(data.meta['PHOTFLAM'], 7.8711728E-20)
             assert 'SCI,1' in data.components
 
             # Request specific extension (name only), use given label
-            parse_data(imviz_helper.app, pf, ext='CTX', show_in_viewer=False)
+            imviz_helper.load_data(pf, ext='CTX', show_in_viewer=False)
             data = imviz_helper.app.data_collection[4]
             assert data.label.startswith('imviz_data|') and data.label.endswith('[CTX,1]')
             assert data.meta['EXTNAME'] == 'CTX'
             assert 'CTX,1' in data.components
 
             # Pass in HDU directly, use given label
-            parse_data(imviz_helper.app, pf[2], data_label='foo', show_in_viewer=False)
+            imviz_helper.load_data(pf[2], data_label='foo', show_in_viewer=False)
             data = imviz_helper.app.data_collection[5]
             assert data.label == 'foo[WHT,1]'
             assert data.meta['EXTNAME'] == 'WHT'
@@ -457,7 +474,7 @@ class TestParseImage:
 
             # Load all extensions
             imviz_helper.app.data_collection.clear()
-            parse_data(imviz_helper.app, filename, ext='*', show_in_viewer=False)
+            imviz_helper.load_data(filename, ext='*', show_in_viewer=False)
             data = imviz_helper.app.data_collection
             assert len(data.labels) == 3
             assert data.labels[0].endswith('[SCI,1]')
@@ -466,13 +483,11 @@ class TestParseImage:
 
         # Cannot load non-image extension
         with pytest.raises(ValueError, match='Imviz cannot load this HDU'):
-            parse_data(imviz_helper.app, filename, ext='HDRTAB',
-                       show_in_viewer=False)
+            parse_data(imviz_helper.app, filename, ext='HDRTAB')
 
         # Invalid FITS extension
         with pytest.raises(KeyError, match='not found'):
-            parse_data(imviz_helper.app, filename, ext='DOES_NOT_EXIST',
-                       data_label='foo', show_in_viewer=False)
+            parse_data(imviz_helper.app, filename, ext='DOES_NOT_EXIST', data_label='foo')
 
 
 def test_load_valid_not_valid(imviz_helper):
