@@ -20,6 +20,19 @@ __all__ = ['MosvizProfileView', 'MosvizImageView', 'MosvizProfile2DView',
 class MosvizProfileView(BqplotProfileView, JdavizViewerMixin):
     default_class = Spectrum1D
 
+    # categories: zoom resets, zoom, pan, subset, select tools, shortcuts
+    tools_nested = [
+                    ['bqplot:home'],
+                    ['jdaviz:boxzoom', 'jdaviz:xrangezoom'],
+                    ['bqplot:panzoom', 'bqplot:panzoom_x', 'bqplot:panzoom_y'],
+                    ['bqplot:xrange'],
+                    ['jdaviz:sidebar_plot', 'jdaviz:sidebar_export']
+                ]
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._initialize_toolbar_nested()
+
     def data(self, cls=None):
         return [layer_state.layer.get_object(cls=cls or self.default_class)
                 for layer_state in self.state.layers
@@ -60,8 +73,21 @@ class MosvizImageView(BqplotImageView, JdavizViewerMixin):
              'bqplot:panzoom', 'bqplot:rectangle',
              'bqplot:circle']
 
+    # categories: zoom resets, zoom, pan, subset, select tools, shortcuts
+    tools_nested = [
+                    ['bqplot:home'],
+                    ['jdaviz:boxzoom'],
+                    ['bqplot:panzoom'],
+                    ['bqplot:circle', 'bqplot:rectangle'],
+                    ['jdaviz:sidebar_plot', 'jdaviz:sidebar_export']
+                ]
+
     default_class = None
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._initialize_toolbar_nested()
+
     def data(self, cls=None):
         return [layer_state.layer  # .get_object(cls=cls or self.default_class)
                 for layer_state in self.state.layers
@@ -95,10 +121,20 @@ class MosvizProfile2DView(BqplotImageView, JdavizViewerMixin):
              'bqplot:panzoom_x',
              'bqplot:xrange']
 
+    # categories: zoom resets, zoom, pan, subset, select tools, shortcuts
+    tools_nested = [
+                    ['bqplot:home'],
+                    ['jdaviz:boxzoom', 'jdaviz:xrangezoom'],
+                    ['bqplot:panzoom', 'bqplot:panzoom_x'],
+                    ['bqplot:xrange'],
+                    ['jdaviz:sidebar_plot', 'jdaviz:sidebar_export']
+                ]
+
     _state_cls = FreezableBqplotImageViewerState
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        self._initialize_toolbar_nested()
         # Setup viewer option defaults
         self.state.aspect = 'auto'
 
@@ -125,6 +161,8 @@ class MosvizTableViewer(TableViewer, JdavizViewerMixin):
         super().__init__(session, *args, **kwargs)
 
         self.figure_widget.observe(self._on_row_selected, names=['highlighted'])
+        # enable scrolling: # https://github.com/glue-viz/glue-jupyter/pull/287
+        self.widget_table.scrollable = True
 
         self._selected_data = {}
         self._shared_image = False
