@@ -1,6 +1,5 @@
 import numpy as np
-
-from bqplot.marks import Lines, Scatter
+import warnings
 
 from glue.core import BaseData
 from glue.core.subset import Subset
@@ -14,8 +13,9 @@ from matplotlib.colors import cnames
 from astropy import units as u
 
 
+from jdaviz.core.events import SpectralMarksChangedMessage
 from jdaviz.core.registries import viewer_registry
-from jdaviz.core.marks import SpectralLine
+from jdaviz.core.marks import SpectralLine, LineUncertainties, ScatterMask
 from jdaviz.core.linelists import load_preset_linelist, get_available_linelists
 from jdaviz.core.freezable_state import FreezableProfileViewerState
 from jdaviz.configs.default.plugins.viewers import JdavizViewerMixin
@@ -33,7 +33,18 @@ class SpecvizProfileView(BqplotProfileView, JdavizViewerMixin):
     tools = ['bqplot:home',
              'jdaviz:boxzoom', 'jdaviz:xrangezoom',
              'bqplot:panzoom', 'bqplot:panzoom_x',
-             'bqplot:panzoom_y', 'bqplot:xrange']
+             'bqplot:panzoom_y', 'bqplot:xrange',
+             'jdaviz:selectline']
+
+    # categories: zoom resets, zoom, pan, subset, select tools, shortcuts
+    tools_nested = [
+                    ['bqplot:home'],
+                    ['jdaviz:xrangezoom', 'jdaviz:boxzoom'],
+                    ['bqplot:panzoom', 'bqplot:panzoom_x', 'bqplot:panzoom_y'],
+                    ['bqplot:xrange'],
+                    ['jdaviz:selectline'],
+                    ['jdaviz:sidebar_plot', 'jdaviz:sidebar_export']
+                ]
 
     default_class = Spectrum1D
     spectral_lines = None
@@ -41,6 +52,7 @@ class SpecvizProfileView(BqplotProfileView, JdavizViewerMixin):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        self._initialize_toolbar_nested(default_tool_priority=['jdaviz:selectslice'])
 
         self.display_uncertainties = False
         self.display_mask = False
@@ -117,12 +129,12 @@ class SpecvizProfileView(BqplotProfileView, JdavizViewerMixin):
             raise ValueError("Line table must have a 'linename' column'")
         if "rest" not in line_table.columns:
             raise ValueError("Line table must have a 'rest' column'")
+        if np.any(line_table['rest'] <= 0):
+            raise ValueError("all rest values must be positive")
 
         # Use the redshift of the displayed spectrum if no redshifts are specified
-        if "redshift" not in line_table.colnames:
-            line_table["redshift"] = self.data()[0].spectral_axis.redshift
-        # Make sure the redshift column is a quantity
-        line_table["redshift"] = u.Quantity(line_table["redshift"])
+        if "redshift" in line_table.colnames:
+            warnings.warn("per line/list redshifts not supported, use viz.set_redshift")
 
         # Set whether to show all of the lines on the plot by default on load
         # We convert bool to int to work around ipywidgets json serialization
@@ -184,6 +196,13 @@ class SpecvizProfileView(BqplotProfileView, JdavizViewerMixin):
         if return_table:
             return line_table
 
+    def _broadcast_plotted_lines(self, marks=None):
+        if marks is None:
+            marks = [x for x in self.figure.marks if isinstance(x, SpectralLine)]
+
+        msg = SpectralMarksChangedMessage(marks, sender=self)
+        self.session.hub.broadcast(msg)
+
     def erase_spectral_lines(self, name=None, name_rest=None, show_none=True):
         """
         Erase either all spectral lines, all spectral lines sharing the same
@@ -195,6 +214,7 @@ class SpecvizProfileView(BqplotProfileView, JdavizViewerMixin):
             fig.marks = [x for x in fig.marks if not isinstance(x, SpectralLine)]
             if show_none:
                 self.spectral_lines["show"] = False
+            self._broadcast_plotted_lines([])
         else:
             temp_marks = []
             # Toggle "show" value in main astropy table. The astropy table
@@ -221,6 +241,7 @@ class SpecvizProfileView(BqplotProfileView, JdavizViewerMixin):
                                 continue
                 temp_marks.append(x)
             fig.marks = temp_marks
+            self._broadcast_plotted_lines()
 
     def get_scales(self):
         fig = self.figure
@@ -252,6 +273,7 @@ class SpecvizProfileView(BqplotProfileView, JdavizViewerMixin):
 
         self.figure.marks = self.figure.marks + [line_mark]
         line["show"] = True
+        self._broadcast_plotted_lines()
 
     def plot_spectral_lines(self, colors=["blue"], **kwargs):
         """
@@ -281,6 +303,7 @@ class SpecvizProfileView(BqplotProfileView, JdavizViewerMixin):
                                 colors=[color], **kwargs)
             marks.append(line)
         fig.marks = fig.marks + marks
+        self._broadcast_plotted_lines()
 
     def available_linelists(self):
         return get_available_linelists()
@@ -297,25 +320,17 @@ class SpecvizProfileView(BqplotProfileView, JdavizViewerMixin):
         self.display_uncertainties = False
         self.display_mask = False
 
-        marks = self.figure.marks[:]
-
         # Remove extra traces, in case they exist.
-        self._clean_error(marks)
-        self._clean_mask(marks)
+        self._clean_error()
+        self._clean_mask()
 
-        self.figure.marks = marks
+    def _clean_mask(self):
+        fig = self.figure
+        fig.marks = [x for x in fig.marks if not isinstance(x, ScatterMask)]
 
-    def _clean_mask(self, marks):
-        if hasattr(self, 'mask_line_mark') and self.mask_line_mark is not None:
-            marks.remove(self.mask_line_mark)
-            self.mask_line_mark = None
-            self.mask_trace_pointer = None
-
-    def _clean_error(self, marks):
-        if hasattr(self, 'error_line_mark') and self.error_line_mark is not None:
-            marks.remove(self.error_line_mark)
-            self.error_line_mark = None
-            self.error_trace_pointer = None
+    def _clean_error(self):
+        fig = self.figure
+        fig.marks = [x for x in fig.marks if not isinstance(x, LineUncertainties)]
 
     def add_data(self, data, color=None, alpha=None, **layer_state):
         """
@@ -340,33 +355,6 @@ class SpecvizProfileView(BqplotProfileView, JdavizViewerMixin):
         # trace representing the spectrum itself.
         result = super().add_data(data, color, alpha, **layer_state)
 
-        # Index of spectrum trace plotted by the super class. It is the
-        # **latest** item in figure.marks that is a Line instance
-        for ind, mark in reversed(list(enumerate(self.figure.marks))):
-            # we'll use __class__.__name__ since other entries (spectral lines,
-            # etc) defined in jdaviz.core.marks inherit from Lines
-            if mark.__class__.__name__ == 'Lines':
-                self.data_trace_pointer = ind
-                break
-        else:  # pragma: no cover
-            raise ValueError("could not find mark for added data")
-
-        self.error_trace_pointer = None
-        self.mask_trace_pointer = None
-
-        # Color and opacity are taken from the already plotted trace,
-        # in case they are not set explicitly by the caller.
-        self._color = self.figure.marks[self.data_trace_pointer].colors[0]
-        if color:
-            self._color = color
-
-        self._alpha = self.figure.marks[self.data_trace_pointer].opacities[0]
-        if alpha:
-            self._alpha = alpha
-
-        # An opacity defined specifically for the shaded areas.
-        self._alpha_shade = self._alpha / 3
-
         self._plot_uncertainties()
 
         self._plot_mask()
@@ -374,84 +362,98 @@ class SpecvizProfileView(BqplotProfileView, JdavizViewerMixin):
         return result
 
     def _plot_mask(self):
+        if not self.display_mask:
+            return
 
-        _data = self._data[0]
-        _data_trace = self.data_trace_pointer
+        # Remove existing mask marks
+        self._clean_mask()
 
-        if 'mask' in _data.components and self.display_mask:
+        # Loop through all active data in the viewer
+        for index, layer_state in enumerate(self.state.layers):
+            comps = [str(component) for component in layer_state.layer.components]
 
-            # If a mask was already plotted, remove it.
-            if (hasattr(self, 'mask_trace_pointer') and
-                    self.mask_trace_pointer is not None):
-                marks = self.figure.marks[:]
-                self._clean_mask(marks)
-                self.figure.marks = marks
+            # Skip subsets
+            if hasattr(layer_state.layer, "subset_state"):
+                continue
 
-            mask = _data['mask'].data
+            # Ignore data that does not have a mask component
+            if "mask" in comps:
+                mask = np.array(layer_state.layer['mask'].data)
 
-            # get trace with spectrum.
-            x = self.figure.marks[_data_trace].x
-            y = self.figure.marks[_data_trace].y
+                data_x = layer_state.layer.data.get_object().spectral_axis
+                data_y = layer_state.layer.data.get_object().flux.value
 
-            # For plotting markers only for the masked data
-            # points, erase un-masked data from trace.
-            y = np.where(np.asarray(mask) == 0, np.nan, y)
+                # For plotting markers only for the masked data
+                # points, erase un-masked data from trace.
+                y = np.where(np.asarray(mask) == 0, np.nan, data_y)
 
-            # there is no 'X' marker option in bqplot
-            self.mask_line_mark = Scatter(scales=self.scales,
-                                          marker='cross',
-                                          x=x,
-                                          y=y,
-                                          stroke_width=0.5,
-                                          # colors=['red'],
-                                          colors=[self._color],
-                                          default_size=25,
-                                          default_opacities=[self._alpha],
-                                          )
-            self.figure.marks = list(self.figure.marks) + [self.mask_line_mark]
-
-            # We added an extra trace. Get pointer to it.
-            self.mask_trace_pointer = len(self.figure.marks) - 1
+                # A subclass of the bqplot Scatter object, ScatterMask places
+                # 'X' marks where there is masked data in the viewer.
+                color = layer_state.color
+                alpha_shade = layer_state.alpha / 3
+                mask_line_mark = ScatterMask(scales=self.scales,
+                                             marker='cross',
+                                             x=data_x,
+                                             y=y,
+                                             stroke_width=0.5,
+                                             colors=[color],
+                                             default_size=25,
+                                             default_opacities=[alpha_shade]
+                                             )
+                # Add mask marks to viewer
+                self.figure.marks = list(self.figure.marks) + [mask_line_mark]
 
     def _plot_uncertainties(self):
+        if not self.display_uncertainties:
+            return
 
-        _data = self._data[0]
-        _data_trace = self.data_trace_pointer
+        # Remove existing error bars
+        self._clean_error()
 
-        if 'uncertainty' in _data.components and self.display_uncertainties:
-            error = np.array(_data['uncertainty'].data)
+        # Loop through all active data in the viewer
+        for index, layer_state in enumerate(self.state.layers):
 
-            # If uncertainties were already plotted, remove them.
-            if (hasattr(self, 'error_trace_pointer') and
-                    self.error_trace_pointer is not None):
-                marks = self.figure.marks[:]
-                self._clean_error(marks)
-                self.figure.marks = marks
+            # Skip subsets
+            if hasattr(layer_state.layer, "subset_state"):
+                continue
 
-            # The shaded band around the spectrum trace is bounded by
-            # two lines, above and below the spectrum trace itself.
-            x = np.array([np.ndarray.tolist(self.figure.marks[_data_trace].x),
-                          np.ndarray.tolist(self.figure.marks[_data_trace].x)])
-            y = np.array([np.ndarray.tolist(self.figure.marks[_data_trace].y - error),
-                          np.ndarray.tolist(self.figure.marks[_data_trace].y + error)])
+            comps = [str(component) for component in layer_state.layer.components]
 
-            # A Lines bqplot instance with two lines and shaded area
-            # in between.
-            self.error_line_mark = Lines(scales=self.scales,
-                                         x=[x],
-                                         y=[y],
-                                         stroke_width=1,
-                                         colors=[self._color, self._color],
-                                         fill_colors=[self._color, self._color],
-                                         opacities=[0.0, 0.0],
-                                         fill_opacities=[self._alpha_shade, self._alpha_shade],
-                                         fill='between',
-                                         close_path=False
-                                         )
-            self.figure.marks = list(self.figure.marks) + [self.error_line_mark]
+            # Ignore data that does not have an uncertainty component
+            if "uncertainty" in comps:  # noqa
+                error = np.array(layer_state.layer['uncertainty'].data)
 
-            # We added an extra trace. Get pointer to it.
-            self.error_trace_pointer = len(self.figure.marks) - 1
+                data_x = layer_state.layer.data.get_object().spectral_axis
+                data_y = layer_state.layer.data.get_object().flux.value
+
+                # The shaded band around the spectrum trace is bounded by
+                # two lines, above and below the spectrum trace itself.
+                x = np.array([np.ndarray.tolist(data_x),
+                              np.ndarray.tolist(data_x)])
+                y = np.array([np.ndarray.tolist(data_y - error),
+                              np.ndarray.tolist(data_y + error)])
+
+                # A subclass of the bqplot Lines object, LineUncertainties keeps
+                # track of uncertainties plotted in the viewer. LineUncertainties
+                # appear with two lines and shaded area in between.
+                color = layer_state.color
+                alpha_shade = layer_state.alpha / 3
+                error_line_mark = LineUncertainties(viewer=self,
+                                                    x=[x],
+                                                    y=[y],
+                                                    scales=self.scales,
+                                                    stroke_width=1,
+                                                    colors=[color, color],
+                                                    fill_colors=[color, color],
+                                                    opacities=[0.0, 0.0],
+                                                    fill_opacities=[alpha_shade,
+                                                                    alpha_shade],
+                                                    fill='between',
+                                                    close_path=False
+                                                    )
+
+                # Add error lines to viewer
+                self.figure.marks = list(self.figure.marks) + [error_line_mark]
 
     def set_plot_axes(self):
         # Get data to be used for axes labels
